/* 
 * Hot potato dest driver for kkv
 */

#include "fridge_test.h"

static void
die(const char *message)
{
	perror(message);
	exit(1);
}

pid_t gettid(void) {
	return syscall(__NR_gettid);
}

void* 
init_hotpotato_thread(void* ignore) 
{
#ifdef VERBOSE
    printf("Thread [%u] initialized\n", gettid()); 
#endif
	int key = 0xbae; 
	char* hp = "hot potato";
	assert(kkv_put(key, hp, strlen(hp) +1, 0) == 0); 
    
	pthread_exit(NULL); 
}

void* 
play_hotpotato_thread(void* ignore)
{
#ifdef VERBOSE
    printf("Thread [%u] initialized\n", gettid()); 
#endif
    int key = 0xbae; 
    char* hp = "hot potato";
    char res[MAX_KEY_SIZE];
    while(kkv_get(key, res, MAX_KEY_SIZE, 0) != 0) {
#ifdef VERBOSE
        printf("[%u] Cold potato!\n", gettid()); 
#endif
    }

#ifdef VERBOSE
    printf("[%u] Hot potato!\n", gettid()); 
#endif

    assert (strcmp(res, hp) == 0); 
    assert(kkv_put(key, res, strlen(res) +1, 0) == 0); 

	pthread_exit(NULL);
}

void
hot_potato(int nthreads)
{
	pthread_t threads[nthreads]; 
	int i; 
	
	if (pthread_create((pthread_t *) threads, NULL, init_hotpotato_thread, NULL) != 0) {
		die("pthread_create() failed"); 
	}

	for (i = 1; i < nthreads; i++) {
		if (pthread_create((threads + i), NULL, play_hotpotato_thread, NULL) != 0) {
			die("pthread_create() failed"); 
		}
	}

	for (i = 0; i < nthreads; i++) {
		pthread_join(threads[i], NULL); 
	}	
}
